@@ -1,4 +1,4 @@
-from typing import List, Dict, Tuple
+from typing import List, Dict, Tuple, Optional
 from itertools import combinations_with_replacement, permutations, product
 import numpy as np
 from .block import Block
@@ -504,183 +504,294 @@ def rotated_periodicity(blocks:List[Block], matched_faces:List[Dict[str,int]], o
         outer_faces_all[j].K *= gcd_to_use
     return periodic_faces_export, outer_faces_export, periodic_faces, outer_faces_all
 
-def translational_periodicity(blocks:List[Block], lower_connected_faces:List[Dict[str,int]], upper_connected_faces:List[Dict[str,int]],delta:float=None, translational_direction:str = "z"):
-    """Find periodicity using translated blocks. Simple example: if you have a rectangle and the top and bottom surfaces are periodic, this will copy the rectangle and shift it up to find which surfaces match. 
+def translational_periodicity(
+    blocks: List[Block],
+    outer_faces: List[Dict[str,int]],
+    delta: float = None,
+    translational_direction: str = "z",
+    node_tol_xyz: float = None,        # global override; if None we compute per-pair adaptively
+    min_shared_frac: float = 0.02,
+    min_shared_abs: int = 4,
+    stride_u: int = 1,
+    stride_v: int = 1,
+    ) -> Tuple[ 
+               List[Dict[str, Dict[str, int]]], 
+               List[Tuple[Face, Face, Dict[str, str]]], 
+               List[Dict[str,int]]
+            ]:
+    """
+    Detect translational periodicity between block faces along a given axis.
+
+    This function takes a set of outer block faces and attempts to identify 
+    periodic counterparts across the domain in the specified translational 
+    direction ('x', 'y', or 'z'). It works by:
+
+      1. **Bounding faces:** Uses `find_bounding_faces` to identify candidate 
+         lower/upper faces for the given axis.
+      2. **Grid reduction:** Reduces blocks to their greatest common divisor 
+         (GCD) resolution to make indexing consistent across blocks.
+      3. **Shifting:** Creates shifted copies of all blocks in both positive 
+         and negative directions along the periodic axis.
+      4. **Precheck in orthogonal plane:** Uses a fast projection test 
+         (orthogonal to the periodic axis) to determine whether two faces 
+         could possibly match. This greatly reduces false negatives when 
+         spacing/tolerances differ slightly.
+      5. **Node-based match:** Calls `Face.touches_by_nodes` on candidate 
+         pairs to check shared node positions, with an adaptive tolerance 
+         based on the in-plane spacing of each face.
+      6. **Pairing:** Records each valid pair of periodic faces, their 
+         IJK index mappings (min→min or min→max), and removes matched faces 
+         from the outer-face list.
+      7. **Scaling back:** Rescales reduced indices back to the original grid 
+         spacing so results are consistent with input block resolution.
 
     Args:
-        blocks (List[Block]): List of blocks for a particular geometry. Do not duplicate the geometry and pass it in
-        lower_connected_faces (List[Dict[str,int]]): List of faces in the lower or left bound of the mesh
-        upper_connected_faces (List[Dict[str,int]]): List of faces in the upper or right bound of the mesh
-        delta (float): if specified the delta will be used to shift the blocks. Defaults to None
-        translational_direction (str, Optional): "x", "y", or "z" 
-    
-    Example:
-        y_periodic_faces_export, periodic_faces = translational_periodicity(blocks,left_bound,right_bound,translational_direction='y')
+        blocks (List[Block]): List of blocks.
+        outer_faces (List[Dict[str,int]]): Outer faces represented as 
+            dictionaries (with IMIN, JMIN, KMIN, IMAX, JMAX, KMAX).
+        delta (float, optional): Periodicity spacing along the chosen axis. 
+            If None, it is inferred from the global block min/max extent.
+        translational_direction (str, optional): Axis to check ('x','y','z'). 
+            Default is 'z'.
+        node_tol_xyz (float, optional): Absolute coordinate tolerance for 
+            node-matching. If None, tolerance is computed adaptively based 
+            on median in-plane spacing of candidate faces.
+        min_shared_frac (float, optional): Minimum fraction of nodes that must 
+            overlap for two faces to be considered periodic. Default 0.02.
+        min_shared_abs (int, optional): Minimum absolute number of shared nodes. 
+            Default 4.
+        stride_u (int, optional): Subsampling stride along the first face index 
+            direction. Default 1 (no skipping).
+        stride_v (int, optional): Subsampling stride along the second face index 
+            direction. Default 1 (no skipping).
 
     Returns:
-        (Tuple): containing
-            
-            - **periodic_faces_export** (List[Dict[str,int]]):  This is list of all the surfaces/faces that match when rotated by an angle formatted as a dictionary.
-            - **outer_faces_export** (List[Dict[str,int]]): These are the list of outer faces that are not periodic formatted as a dictionary.
-            - **periodic_faces** (List[Tuple[Face,Face]]): - This is a list of Face objects that are connected to each other organized as a list of tuples: [Face1, Face2] where Face 1 will contain the block number and the diagonals [IMIN,JMIN,KMIN,IMAX,JMAX,KMAX]. Example: blk: 1 [168,0,0,268,100,0].
-            - **outer_faces_all** (List[Face]): This is a list of outer faces save as a list of Faces
+        Tuple[
+            List[Dict[str, Dict[str,int]]], 
+            List[Tuple[Face, Face, Dict[str,str]]], 
+            List[Dict[str,int]]
+        ]:
+            - **periodic_faces_export**: Export-ready dictionaries describing 
+              each periodic pair (block indices, face extents, index mapping, 
+              and match mode).
+            - **periodic_pairs**: Matched periodic face pairs as `Face` objects 
+              with IJK mapping.
+            - **outer_faces_remaining**: Updated list of outer faces with 
+              periodic ones removed (preserving any existing `id` fields).
+
+    Notes:
+        - Works for periodicity in **x**, **y**, or **z** directions.
+        - The adaptive tolerance makes the method robust to small spacing 
+          differences between blocks.
+        - The orthogonal-plane precheck avoids expensive node comparisons 
+          when faces clearly do not align.
     """
-    gcd_array = list()
-    # Find the gcd of all the blocks
-    for block_indx in range(len(blocks)):
-        block = blocks[block_indx]
-        gcd_array.append(math.gcd(block.IMAX-1, math.gcd(block.JMAX-1, block.KMAX-1)))
-    gcd_to_use = min(gcd_array) # You need to use the minimum gcd otherwise 1 block may not exactly match the next block. They all have to be scaled the same way.
-    lower_connected_faces = outer_face_dict_to_list(blocks,lower_connected_faces,gcd_to_use)
-    upper_connected_faces = outer_face_dict_to_list(blocks,upper_connected_faces,gcd_to_use)
-    blocks = reduce_blocks(deepcopy(blocks),gcd_to_use)    
-    
-    # Now for the periodicity part
-    
-    def shift_blocks(sign:int=1):
-        blocks_shifted = deepcopy(blocks)
-        if translational_direction.lower().strip() == "x":
-            xmin = min([b.X.min() for b in blocks])
-            xmax = max([b.X.max() for b in blocks])
-            dx = xmax-xmin if not delta else delta
-            [b.shift(sign*dx,translational_direction) for b in blocks_shifted]
-        elif translational_direction.lower().strip() == "y":
-            ymin = np.array([(b.X.min(), b.Y.min(), b.Z.min()) for b in blocks]) # Look at the front face 
-            ymax = np.array([(b.X.min(), b.Y.max(), b.Z.min()) for b in blocks])
-            xmin = min([b.X.min() for b in blocks])
-            zmin = min([b.Z.min() for b in blocks])
+    # 0) lower/upper via your finder (dicts at original scale)
+    lower_connected_faces, upper_connected_faces, _, _ = find_bounding_faces(
+        blocks, outer_faces, translational_direction, "both"
+    )
 
-            ymin = ymin[ymin[:,0] == xmin,:]
-            ymax = ymax[ymax[:,0] == xmin,:]
+    axis = translational_direction.lower().strip()
+    assert axis in ("x","y","z")
 
-            ymin = ymin[ymin[:,2] == zmin,:]
-            ymax = ymax[ymax[:,2] == zmin,:]
-            
-            dy = ymax[:,1].max() - ymin[:,1].min() if not delta else delta
-            [b.shift(sign*dy,translational_direction) for b in blocks_shifted]
-        else: #  direction.lower().strip() == "z"
-            zmin = min([b.Z.min() for b in blocks])
-            zmax = max([b.Z.max() for b in blocks])
-            dz = zmax-zmin if not delta else delta
-            [b.shift(sign*dz,translational_direction) for b in blocks_shifted]
-        return blocks_shifted
+    # 1) GCD reduce
+    gcd_array = [math.gcd(b.IMAX-1, math.gcd(b.JMAX-1, b.KMAX-1)) for b in blocks]
+    gcd_to_use = min(gcd_array)
 
-    blocks_shifted = shift_blocks()
-    periodic_found = True # start of the loop 
-    
-    # Here we make a list of all the outer faces
-    times_failed = 0 
-    periodic_faces = list()      # This is the output of the code 
-    periodic_faces_export = list()
-    non_matching = list()
-    lower_connected_faces = list(set(lower_connected_faces))
-    upper_connected_faces = list(set(upper_connected_faces))
-    lower_blocks = [l.BlockIndex for l in lower_connected_faces]
-    upper_blocks = [u.BlockIndex for u in upper_connected_faces]
-    pbar = tqdm(total = len(lower_connected_faces))
-    
-    periodicity_tol = 1E-6 
-    while len(lower_connected_faces)>0:
-        periodic_found = False
-        face1 = lower_connected_faces[0]
-        for indx in range(len(upper_connected_faces)):
-            face2 = upper_connected_faces[indx]
-            # Check if surfaces are periodic with each other
-            pbar.set_description(f"Checking connections block {face1.blockIndex} with {face2.blockIndex}")
-            # Shift block 1 -> Check periodicity -> if not periodic -> shift Block 1 opposite direction -> Check periodicity
-            #   Rotate Block 1
-            block1 = blocks[face1.blockIndex]
-            block1_shifted = blocks_shifted[face1.blockIndex]
-            block2 = blocks[face2.blockIndex]
-            block2_shifted = blocks_shifted[face2.blockIndex]
+    lower_faces_r = outer_face_dict_to_list(blocks, lower_connected_faces, gcd_to_use)
+    upper_faces_r = outer_face_dict_to_list(blocks, upper_connected_faces, gcd_to_use)
+    blocks_r = reduce_blocks(deepcopy(blocks), gcd_to_use)
 
-            #   Check periodicity
-            _, periodic_faces_temp, split_faces_temp = __periodicity_check__(face1,face2,block1_shifted, block2,periodicity_tol)                           
-
-            if len(periodic_faces_temp) > 0:
-                lower_connected_faces.pop(0)
-                upper_connected_faces.pop(indx)
-                periodic_faces.append(periodic_faces_temp)
-                periodic_faces_export.append(face_matches_to_dict(periodic_faces_temp[0],periodic_faces_temp[1],block1_shifted,block2))
-                lower_split_faces = [s for s in split_faces_temp if s.BlockIndex in lower_blocks]
-                upper_split_faces = [s for s in split_faces_temp if s.BlockIndex in upper_blocks]
-                lower_connected_faces.extend(lower_split_faces)
-                upper_connected_faces.extend(upper_split_faces)
-                periodic_found = True
-                pbar.update(1)
-                break
-            else:
-                # Try the other way 
-                _, periodic_faces_temp, split_faces_temp = __periodicity_check__(face1,face2,block1,block2_shifted,periodicity_tol)
-                if len(periodic_faces_temp) > 0:
-                    lower_connected_faces.pop(0)
-                    upper_connected_faces.pop(indx)
-                    periodic_faces.append(periodic_faces_temp)
-                    periodic_faces_export.append(face_matches_to_dict(periodic_faces_temp[0],periodic_faces_temp[1],block1, block2_shifted))
-                    lower_split_faces = [s for s in split_faces_temp if s.BlockIndex in lower_blocks]
-                    upper_split_faces = [s for s in split_faces_temp if s.BlockIndex in upper_blocks]
-                    lower_connected_faces.extend(lower_split_faces)
-                    upper_connected_faces.extend(upper_split_faces)
-                    periodic_found = True
-                    pbar.update(1)
-                    break
-    
-        if periodic_found == False:
-            # Lets switch the order 
-            non_matching.append(deepcopy(face1))
-            lower_connected_faces.pop(0)
-            # periodicity_tol *=10
-            times_failed+=1
-
-    if len(non_matching)>0:
-        print(f"\nNot periodic {translational_direction}")
+    # 2) Δ along axis (if not provided)
+    if axis == "x":
+        a_min = min(b.X.min() for b in blocks_r); a_max = max(b.X.max() for b in blocks_r)
+    elif axis == "y":
+        a_min = min(b.Y.min() for b in blocks_r); a_max = max(b.Y.max() for b in blocks_r)
     else:
-        print(f"\nPeriodic {translational_direction}")
-        
-    # remove any duplicate periodic face pairs 
-    indx_to_remove = list()
-    for i in range(len(periodic_faces)):
-        for j in range(i+1,len(periodic_faces)):
-            if periodic_faces[i][0] == periodic_faces[j][0]:
-                if periodic_faces[i][1] == periodic_faces[j][1]:
-                    indx_to_remove.append(j)
-            if periodic_faces[i][1] == periodic_faces[j][0]:
-                if periodic_faces[i][0] == periodic_faces[j][1]:
-                    indx_to_remove.append(j)
-    
+        a_min = min(b.Z.min() for b in blocks_r); a_max = max(b.Z.max() for b in blocks_r)
+    d_axis = (a_max - a_min) if (delta is None) else float(delta)
 
-    periodic_faces_export = [periodic_faces_export[i] for i in range(len(periodic_faces)) if i not in indx_to_remove]
-    periodic_faces = [periodic_faces[i] for i in range(len(periodic_faces)) if i not in indx_to_remove]
+    # 3) Shifted copies
+    def shift_blocks(bb: List[Block], amount: float) -> List[Block]:
+        cp = deepcopy(bb)
+        for b in cp:
+            b.shift(amount, axis)
+        return cp
 
-    # scale it up
-    for i in range(len(periodic_faces_export)):
-        periodic_faces_export[i]['block1']['IMIN'] *= gcd_to_use
-        periodic_faces_export[i]['block1']['JMIN'] *= gcd_to_use
-        periodic_faces_export[i]['block1']['KMIN'] *= gcd_to_use
-        periodic_faces_export[i]['block1']['IMAX'] *= gcd_to_use
-        periodic_faces_export[i]['block1']['JMAX'] *= gcd_to_use
-        periodic_faces_export[i]['block1']['KMAX'] *= gcd_to_use
+    blocks_up = shift_blocks(blocks_r, +d_axis)
+    blocks_dn = shift_blocks(blocks_r, -d_axis)
 
-        if (periodic_faces_export[i]['block2']['IMIN'] == 168):
-            print("check")
-        periodic_faces_export[i]['block2']['IMIN'] *= gcd_to_use
-        periodic_faces_export[i]['block2']['JMIN'] *= gcd_to_use
-        periodic_faces_export[i]['block2']['KMIN'] *= gcd_to_use
-        periodic_faces_export[i]['block2']['IMAX'] *= gcd_to_use
-        periodic_faces_export[i]['block2']['JMAX'] *= gcd_to_use
-        periodic_faces_export[i]['block2']['KMAX'] *= gcd_to_use
-    
-    for i in range(len(periodic_faces)):
-        periodic_faces[i][0].I *= gcd_to_use
-        periodic_faces[i][0].J *= gcd_to_use
-        periodic_faces[i][0].K *= gcd_to_use
+    def B(which: str, idx: int) -> Block:
+        return {"orig": blocks_r, "up": blocks_up, "dn": blocks_dn}[which][idx]
 
-        periodic_faces[i][1].I *= gcd_to_use
-        periodic_faces[i][1].J *= gcd_to_use
-        periodic_faces[i][1].K *= gcd_to_use
+    # 4) Helpers for adaptive tolerance
+    def _median_inplane_spacing(face: Face, block: Block) -> float:
+        """Median edge length on the face (in-plane)."""
+        I0,I1,J0,J1,K0,K1 = face.IMIN,face.IMAX,face.JMIN,face.JMAX,face.KMIN,face.KMAX
+        X,Y,Z = block.X, block.Y, block.Z
+        if face.const_type == 0:  # I const → vary (J,K)
+            i = I0
+            x = X[i,J0:J1+1,K0:K1+1]; y = Y[i,J0:J1+1,K0:K1+1]; z = Z[i,J0:J1+1,K0:K1+1]
+        elif face.const_type == 1:  # J const → vary (I,K)
+            j = J0
+            x = X[I0:I1+1,j,K0:K1+1]; y = Y[I0:I1+1,j,K0:K1+1]; z = Z[I0:I1+1,j,K0:K1+1]
+        else:  # K const → vary (I,J)
+            k = K0
+            x = X[I0:I1+1,J0:J1+1,k]; y = Y[I0:I1+1,J0:J1+1,k]; z = Z[I0:I1+1,J0:J1+1,k]
+        s = []
+        if x.shape[0] > 1:
+            dx = np.diff(x, axis=0); dy = np.diff(y, axis=0); dz = np.diff(z, axis=0)
+            s.append(np.sqrt(dx*dx + dy*dy + dz*dz))
+        if x.shape[1] > 1:
+            dx = np.diff(x, axis=1); dy = np.diff(y, axis=1); dz = np.diff(z, axis=1)
+            s.append(np.sqrt(dx*dx + dy*dy + dz*dz))
+        if not s: return 1.0
+        return float(np.median(np.concatenate([v.ravel() for v in s])))
 
-    return periodic_faces_export, periodic_faces
+    def _pair_tol(fA: Face, fB: Face) -> float:
+        """Adaptive absolute tolerance per pair (use global override if provided)."""
+        if node_tol_xyz is not None:
+            return float(node_tol_xyz)
+        sA = _median_inplane_spacing(fA, B("orig", fA.BlockIndex))
+        sB = _median_inplane_spacing(fB, B("orig", fB.BlockIndex))
+        # ~3% of local in-plane spacing; floor at 1e-4 (tune if needed)
+        return max(0.03 * max(sA, sB), 1e-4)
 
+    # 5) General orthogonal-plane precheck (works for x/y/z periodicity)
+    def _orthogonal_precheck(fA: Face, fB: Face, bA: Block, bB: Block,
+                             d_axis_local: float, tol: float, axis_local: str) -> bool:
+        """
+        Shift face A along 'axis_local' by d_axis_local, then compare projections onto the
+        orthogonal plane within tolerance. Requires both absolute and fractional overlap.
+        """
+        PA = fA.grid_points(bA, stride_u=1, stride_v=1)
+        PB = fB.grid_points(bB, stride_u=1, stride_v=1)
+        if PA.size == 0 or PB.size == 0:
+            return False
+
+        if axis_local == "x":
+            PA[:,0] += d_axis_local
+            projA, projB = PA[:,1:], PB[:,1:]          # (y,z)
+        elif axis_local == "y":
+            PA[:,1] += d_axis_local
+            projA, projB = PA[:,[0,2]], PB[:,[0,2]]    # (x,z)
+        else:  # "z"
+            PA[:,2] += d_axis_local
+            projA, projB = PA[:,:2], PB[:,:2]          # (x,y)
+
+        QA = np.round(projA / tol).astype(np.int64)
+        QB = np.round(projB / tol).astype(np.int64)
+        if not QA.flags["C_CONTIGUOUS"]: QA = np.ascontiguousarray(QA)
+        if not QB.flags["C_CONTIGUOUS"]: QB = np.ascontiguousarray(QB)
+        vA = QA.view([('', QA.dtype)] * QA.shape[1]).reshape(-1)
+        vB = QB.view([('', QB.dtype)] * QB.shape[1]).reshape(-1)
+        inter = np.intersect1d(vA, vB, assume_unique=False)
+        return inter.size >= max(min_shared_abs, int(min_shared_frac * min(len(vA), len(vB))))
+
+    # 6) Node-sharing matcher using per-pair tol + precheck
+    def faces_match(fL: Face, fU: Face) -> Tuple[bool, str]:
+        bl, bu = fL.BlockIndex, fU.BlockIndex
+        tol_pair = _pair_tol(fL, fU)
+
+        # Fast precheck on orthogonal plane (lower up vs upper orig)
+        if _orthogonal_precheck(fL, fU, B("orig", bl), B("orig", bu), d_axis, tol_pair, axis):
+            return True, f"{axis}_precheck_lower_up"
+
+        # lower moved up vs upper orig
+        if fL.touches_by_nodes(fU, B("up", bl), B("orig", bu),
+                               tol_xyz=tol_pair, min_shared_frac=min_shared_frac,
+                               min_shared_abs=min_shared_abs, stride_u=stride_u, stride_v=stride_v):
+            return True, "lower_up_vs_upper_orig"
+
+        # lower orig vs upper moved down
+        if fL.touches_by_nodes(fU, B("orig", bl), B("dn", bu),
+                               tol_xyz=tol_pair, min_shared_frac=min_shared_frac,
+                               min_shared_abs=min_shared_abs, stride_u=stride_u, stride_v=stride_v):
+            return True, "lower_orig_vs_upper_dn"
+
+        # Symmetric guards
+        if _orthogonal_precheck(fU, fL, B("orig", bu), B("orig", bl), d_axis, tol_pair, axis):
+            return True, f"{axis}_precheck_upper_up"
+        if fU.touches_by_nodes(fL, B("up", bu), B("orig", bl),
+                               tol_xyz=tol_pair, min_shared_frac=min_shared_frac,
+                               min_shared_abs=min_shared_abs, stride_u=stride_u, stride_v=stride_v):
+            return True, "upper_up_vs_lower_orig"
+        if fU.touches_by_nodes(fL, B("orig", bu), B("dn", bl),
+                               tol_xyz=tol_pair, min_shared_frac=min_shared_frac,
+                               min_shared_abs=min_shared_abs, stride_u=stride_u, stride_v=stride_v):
+            return True, "upper_orig_vs_lower_dn"
+
+        return False, ""
+
+    # 7) Index mapping
+    def mapping_minmax(fA: Face, fB: Face) -> Dict[str, str]:
+        out = {}
+        for ax in ("I","J","K"):
+            Amin, Amax = getattr(fA, ax+"MIN"), getattr(fA, ax+"MAX")
+            Bmin, Bmax = getattr(fB, ax+"MIN"), getattr(fB, ax+"MAX")
+            if (Amin == Bmin) and (Amax == Bmax): out[ax] = "min->min"
+            elif (Amin == Bmax) and (Amax == Bmin): out[ax] = "min->max"
+            else:
+                d_mm = abs(Amin-Bmin)+abs(Amax-Bmax)
+                d_mM = abs(Amin-Bmax)+abs(Amax-Bmin)
+                out[ax] = "min->min" if d_mm <= d_mM else "min->max"
+        return out
+
+    # 8) Greedy pairing and export
+    lower_pool = list(dict.fromkeys(lower_faces_r))
+    upper_pool = list(dict.fromkeys(upper_faces_r))
+    periodic_pairs_r: List[Tuple[Face, Face, Dict[str,str]]] = []
+    periodic_export: List[Dict[str, Dict[str,int]]] = []
+
+    for fL in list(lower_pool):
+        for j, fU in enumerate(upper_pool):
+            ok, mode = faces_match(fL, fU)
+            if ok:
+                m = mapping_minmax(fL, fU)
+                periodic_pairs_r.append((fL, fU, m))
+                periodic_export.append({
+                    "block1": {"block_index": fL.BlockIndex,
+                               "IMIN": fL.IMIN, "JMIN": fL.JMIN, "KMIN": fL.KMIN,
+                               "IMAX": fL.IMAX, "JMAX": fL.JMAX, "KMAX": fL.KMAX},
+                    "block2": {"block_index": fU.BlockIndex,
+                               "IMIN": fU.IMIN, "JMIN": fU.JMIN, "KMIN": fU.KMIN,
+                               "IMAX": fU.IMAX, "JMAX": fU.JMAX, "KMAX": fU.KMAX},
+                    "mapping": m,
+                    "mode": mode
+                    }) # type: ignore  
+                upper_pool.pop(j)
+                break  # move to next fL
+
+    # 9) scale back up
+    for rec in periodic_export:
+        for side in ("block1","block2"):
+            for k in ("IMIN","JMIN","KMIN","IMAX","JMAX","KMAX"):
+                rec[side][k] = int(rec[side][k] * gcd_to_use)
+
+    periodic_pairs: List[Tuple[Face, Face, Dict[str,str]]] = []
+    for (fL, fU, m) in periodic_pairs_r:
+        gL = deepcopy(fL); gU = deepcopy(fU)
+        gL.I *= gcd_to_use; gL.J *= gcd_to_use; gL.K *= gcd_to_use
+        gU.I *= gcd_to_use; gU.J *= gcd_to_use; gU.K *= gcd_to_use
+        periodic_pairs.append((gL, gU, m))
+
+    # 10) remove periodic from outer_faces (keep 'id' on remaining)
+    periodic_keys = set()
+    for rec in periodic_export:
+        for side in ("block1","block2"):
+            bi = rec[side]["block_index"]
+            key = (bi, rec[side]["IMIN"], rec[side]["JMIN"], rec[side]["KMIN"],
+                        rec[side]["IMAX"], rec[side]["JMAX"], rec[side]["KMAX"])
+            periodic_keys.add(key)
+
+    outer_faces_remaining = []
+    for o in outer_faces:
+        key = (o["block_index"], o["IMIN"], o["JMIN"], o["KMIN"],
+                               o["IMAX"], o["JMAX"], o["KMAX"])
+        if key not in periodic_keys:
+            outer_faces_remaining.append(o)
+
+    return periodic_export, periodic_pairs, outer_faces_remaining
 
 def linear_real_transform(face1:Face,face2:Face) -> Tuple:
     """Computes the rotation angle from Face1 to Face2. This can be used to check if the faces are periodic 
